#!/usr/bin/python
#
# 1. Create a folder daca2 in your HOME folder
# 2. Put cppcheck-O2 in daca2. It should be built with all optimisations.
# 3. Optional: Put a file called "suppressions.txt" in the daca2 folder.
# 4. Optional: tweak FTPSERVER and FTPPATH in this script below.
# 5. Run the daca2 script:  python daca2.py FOLDER

import subprocess
import sys
import shutil
import glob
import os
import datetime
import time

DEBIAN = ['ftp://ftp.sunet.se/pub/Linux/distributions/Debian/debian/',
          'http://ftp.sunet.se/pub/Linux/distributions/Debian/debian/',
          'ftp://ftp.debian.org/debian/']


def wget(filepath):
    filename = filepath
    if filepath.find('/') >= 0:
        filename = filename[filename.rfind('/') + 1:]
    for d in DEBIAN:
        subprocess.call(
            ['nice', 'wget', '--tries=10', '--timeout=300', '-O', filename, d + filepath])
        if os.path.isfile(filename):
            return True
        print('Sleep for 10 seconds..')
        time.sleep(10)
    return False


def getpackages(folder):
    if not wget('ls-lR.gz'):
        return []
    subprocess.call(['nice', 'gunzip', 'ls-lR.gz'])
    f = open('ls-lR', 'rt')
    lines = f.readlines()
    f.close()
    subprocess.call(['rm', 'ls-lR'])

    path = None
    archives = []
    filename = None
    for line in lines:
        line = line.strip()
        if len(line) < 4:
            if filename:
                archives.append(path + '/' + filename)
            path = None
            filename = None
        elif line[:13 + len(folder)] == './pool/main/' + folder + '/':
            path = line[2:-1]
        elif path and line.find('.orig.tar.') > 0:
            filename = line[1 + line.rfind(' '):]

    for a in archives:
        print(a)

    return archives


def handleRemoveReadonly(func, path, exc):
    import stat
    if not os.access(path, os.W_OK):
        # Is the error an access error ?
        os.chmod(path, stat.S_IWUSR)
        func(path)
    else:
        raise


def removeAllExceptResults():
    count = 5
    while count > 0:
        count = count - 1

        filenames = []
        for g in glob.glob('[A-Za-z0-9]*'):
            filenames.append(g)
        for g in glob.glob('.[a-z]*'):
            filenames.append(g)

        try:
            for filename in filenames:
                if os.path.isdir(filename):
                    shutil.rmtree(filename, onerror=handleRemoveReadonly)
                elif filename != 'results.txt':
                    os.remove(filename)
        except WindowsError, err:
            time.sleep(30)
            if count == 0:
                print('Failed to cleanup files/folders')
                print(err)
                sys.exit(1)
            continue
        except OSError, err:
            time.sleep(30)
            if count == 0:
                print('Failed to cleanup files/folders')
                print(err)
                sys.exit(1)
            continue
        count = 0


def removeLargeFiles(path):
    for g in glob.glob(path + '*'):
        if g == '.' or g == '..':
            continue
        if os.path.islink(g):
            continue
        if os.path.isdir(g):
            removeLargeFiles(g + '/')
        elif os.path.isfile(g) and g[-4:] != '.txt':
            statinfo = os.stat(g)
            if path.find('/clang/INPUTS/') > 0 or statinfo.st_size > 100000:
                os.remove(g)


def scanarchive(filepath):
    # remove all files/folders except results.txt
    removeAllExceptResults()

    results = open('results.txt', 'at')
    results.write(DEBIAN[0] + filepath + '\n')
    results.close()

    if not wget(filepath):
        if not wget(filepath):
            results = open('results.txt', 'at')
            results.write('wget failed\n')
            results.close()
            return

    filename = filepath[filepath.rfind('/') + 1:]
    if filename[-3:] == '.gz':
        subprocess.call(['tar', 'xzvf', filename])
    elif filename[-3:] == '.xz':
        subprocess.call(['tar', 'xJvf', filename])
    elif filename[-4:] == '.bz2':
        subprocess.call(['tar', 'xjvf', filename])

    if filename[:5] == 'flite' or filename[:5] == 'boost' or filename[:6] == 'iceowl' or filename[:7] == 'insight':
        results = open('results.txt', 'at')
        results.write('fixme: skipped package to avoid hang\n')
        results.close()
        return

    removeLargeFiles('')

    print('cppcheck ' + filename)

    p = subprocess.Popen(
        ['nice',
         '../cppcheck-O2',
         '-D__GCC__',
         '--enable=style',
         '--error-exitcode=0',
         '--suppressions-list=../suppressions.txt',
         '.'],
        stdout=subprocess.PIPE,
        stderr=subprocess.PIPE)
    comm = p.communicate()

    results = open('results.txt', 'at')
    if p.returncode == 0:
        results.write(comm[1])
    elif comm[0].find('cppcheck: error: could not find or open any of the paths given.') < 0:
        results.write('Exit code is not zero! Crash?\n')
    results.write('\n')
    results.close()

FOLDER = None
REV = None
for arg in sys.argv[1:]:
    if arg[:6] == '--rev=':
        REV = arg[6:]
    else:
        FOLDER = arg

if not FOLDER:
    print('no folder given')
    sys.exit(1)

archives = getpackages(FOLDER)
if len(archives) == 0:
    print('failed to load packages')
    sys.exit(1)

print('Sleep for 10 seconds..')
time.sleep(10)

workdir = os.path.expanduser('~/daca2/')

print('~/daca2/suppressions.txt')
if not os.path.isfile(workdir + 'suppressions.txt'):
    suppressions = open(workdir + 'suppressions.txt', 'wt')
    suppressions.write('\n')
    suppressions.close()

print('~/daca2/' + FOLDER)
if not os.path.isdir(workdir + FOLDER):
    os.makedirs(workdir + FOLDER)
os.chdir(workdir + FOLDER)

try:
    results = open('results.txt', 'wt')
    results.write('STARTDATE ' + str(datetime.date.today()) + '\n')
    if REV:
        results.write('GIT-REVISION ' + REV + '\n')
    results.write('\n')
    results.close()

    for archive in archives:
        scanarchive(archive)

    results = open('results.txt', 'at')
    results.write('DATE ' + str(datetime.date.today()) + '\n')
    results.close()

except EOFError:
    pass

# remove all files/folders except results.txt
removeAllExceptResults()



import os
import sys
import subprocess


def readdate(data):
    datepos = -1
    if data[:5] == 'DATE ':
        datepos = 0
    else:
        datepos = data.find('\nDATE ')
        if datepos >= 0:
            datepos = datepos + 1

    if datepos < 0:
        return None

    datestr = ''
    datepos = datepos + 5
    while True:
        if datepos >= len(data):
            return None
        d = data[datepos]
        if d >= '0' and d <= '9':
            datestr = datestr + d
        elif d == '\n':
            if len(datestr) == 8:
                return datestr[:4] + '-' + datestr[4:6] + '-' + datestr[6:]
            return None
        elif d != ' ' and d != '-':
            return None
        datepos = datepos + 1

path = '.'
if len(sys.argv) == 2:
    path = sys.argv[1]

mainpage = open(path + '/daca2.html', 'wt')
mainpage.write('<!DOCTYPE html>\n')
mainpage.write('<html lang="en">\n')
mainpage.write('<head>\n')
mainpage.write('<meta charset="utf-8">\n')
mainpage.write('<title>DACA2</title>\n')
mainpage.write('<link rel="stylesheet" href="/site/css/daca2.css">\n')
mainpage.write('<script src="/site/js/sorttable.min.js"></script>\n')
mainpage.write('</head>\n')
mainpage.write('<body>\n')
mainpage.write('<h1>DACA2</h1>\n')
mainpage.write('<p>Results when running latest (git head) Cppcheck on Debian.</p>\n')
mainpage.write('<p>For performance reasons the analysis is limited. Files larger than 100kb are skipped. If analysis of a file takes more than 10 minutes it may be stopped.</p>\n')
mainpage.write('<table class="sortable">\n')
mainpage.write(
    '<tr>' +
    '<th>Name</th>' +
    '<th>Date</th>' +
    '<th>Error</th>' +
    '<th>Warning</th>' +
    '<th>Performance</th>' +
    '<th>Portability</th>' +
    '<th>Style</th>' +
    '<th>Crashes</th>' +
    '<th>VarID 0</th></tr>\n')

lastupdate = None
recent = []

daca2 = os.path.expanduser('~/daca2/')
for lib in range(2):
    for a in "0123456789abcdefghijklmnopqrstuvwxyz":
        if lib == 1:
            a = "lib" + a
        if os.path.isfile(daca2 + a + '/results.txt'):
            f = open(daca2 + a + '/results.txt', 'rt')
            data = f.read()
            f.close()

            datestr = readdate(data)

            if os.path.isfile(daca2 + 'results-' + a + '.txt'):
                f2 = open(daca2 + 'results-' + a + '.txt')
                data2 = f2.read()
                f2.close()

                datestr2 = readdate(data2)
                if not datestr or datestr < datestr2:
                    data = data2
                    datestr = datestr2

            if datestr:
                if not lastupdate or datestr > lastupdate:
                    lastupdate = datestr
                    recent = []
                if datestr == lastupdate:
                    recent.append(a)
            else:
                datestr = ''

            mainpage.write(
                '<tr>' +
                '<td><a href="daca2-' + a + '.html">' + a + '</a></td>' +
                '<td>' + datestr + '</td>' +
                '<td>' + str(data.count('(error)')) + '</td>' +
                '<td>' + str(data.count('(warning)')) + '</td>' +
                '<td>' + str(data.count('(performance)')) + '</td>' +
                '<td>' + str(data.count('(portability)')) + '</td>' +
                '<td>' + str(data.count('(style)')) + '</td>' +
                '<td>' + str(data.count('Crash?')) + '</td>' +
                '<td>' + str(data.count('with varid 0.')) + '</td>' +
                '</tr>\n')

            data = data.replace('&', '&amp;')
            data = data.replace('<', '&lt;')
            data = data.replace('>', '&gt;')
            data = data.replace('\n', '\n')

            f = open(path + '/daca2-' + a + '.html', 'wt')
            f.write('<!DOCTYPE html>\n')
            f.write('<html lang="en">\n')
            f.write('<head>\n')
            f.write('<meta charset="utf-8">\n')
            f.write('<title>DACA2 - ' + a + '</title>\n')
            f.write('</head>\n')
            f.write('<body>\n')
            f.write('<h1>DACA2 - ' + a + '</h1>')
            f.write('<pre>\n' + data + '</pre>\n')
            f.write('</body>\n')
            f.write('</html>\n')
            f.close()

mainpage.write('</table>\n')

if lastupdate:
    mainpage.write('<p>Last update: ' + lastupdate + '</p>')
    allrecent = ''
    for r in recent:
        allrecent = allrecent + ' <a href="daca2-' + r + '.html">' + r + '</a>'
    mainpage.write('<p>Most recently updated:' + allrecent + '</p>')

mainpage.write('</body>\n')
mainpage.write('</html>\n')



#!/usr/bin/python
#
# Cppcheck - A tool for static C/C++ code analysis
# Copyright (C) 2007-2014 Daniel Marjamaeki and Cppcheck team.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import unittest
import matchcompiler


class MatchCompilerTest(unittest.TestCase):

    def setUp(self):
        self.mc = matchcompiler.MatchCompiler(verify_mode=False)

    def test_parseMatch(self):
        self.assertEqual(self.mc.parseMatch('  Token::Match(tok, ";") ', 2), [
                         'Token::Match(tok, ";")', 'tok', ' ";"'])
        self.assertEqual(self.mc.parseMatch('  Token::Match(tok,', 2), None)
                         # multiline Token::Match is not supported yet
        self.assertEqual(self.mc.parseMatch('  Token::Match(Token::findsimplematch(tok,")"), ";")', 2), [
                         'Token::Match(Token::findsimplematch(tok,")"), ";")', 'Token::findsimplematch(tok,")")', ' ";"'])  # inner function call

    def test_replaceTokenMatch(self):
        input = 'if (Token::Match(tok, "foobar")) {'
        output = self.mc._replaceTokenMatch(input)
        self.assertEqual(output, 'if (match1(tok)) {')
        self.assertEqual(1, len(self.mc._matchStrs))
        self.assertEqual(1, self.mc._matchStrs['foobar'])

        input = 'if (Token::Match(tok->next()->next(), "foobar %type% %num%")) {'
        output = self.mc._replaceTokenMatch(input)
        self.assertEqual(output, 'if (match2(tok->next()->next())) {')
        self.assertEqual(1, len(self.mc._matchStrs))

        input = 'if (Token::Match(tok, "foo\"special\"bar %num%")) {'
        output = self.mc._replaceTokenMatch(input)
        # FIXME: Currently detected as non-static pattern
        self.assertEqual(
            output, 'if (Token::Match(tok, "foo"special"bar %num%")) {')
        # self.assertEqual(3, len(self.mc._matchStrs))

    def test_replaceTokenMatchWithVarId(self):
        input = 'if (Token::Match(tok, "foobar %varid%", 123)) {'
        output = self.mc._replaceTokenMatch(input)
        self.assertEqual(output, 'if (match1(tok, 123)) {')
        self.assertEqual(1, len(self.mc._matchStrs))

        input = 'if (Token::Match(tok->next()->next(), "%varid% foobar", tok->varId())) {'
        output = self.mc._replaceTokenMatch(input)
        self.assertEqual(
            output, 'if (match2(tok->next()->next(), tok->varId())) {')
        self.assertEqual(1, len(self.mc._matchStrs))

        input = 'if (Token::Match(tok, "foo\"special\"bar %type% %varid%", my_varid_cache)) {'
        output = self.mc._replaceTokenMatch(input)
        # FIXME: Currently detected as non-static pattern
        self.assertEqual(
            output, 'if (Token::Match(tok, "foo"special"bar %type% %varid%", my_varid_cache)) {')
        # self.assertEqual(1, len(self.mc._matchStrs))

        # test caching: reuse existing matchX()
        input = 'if (Token::Match(tok, "foobar %varid%", 123)) {'
        output = self.mc._replaceTokenMatch(input)
        self.assertEqual(output, 'if (match1(tok, 123)) {')
        self.assertEqual(1, len(self.mc._matchStrs))

        # two in one line
        input = 'if (Token::Match(tok, "foobar2 %varid%", 123) || Token::Match(tok, "%type% %varid%", 123)) {'
        output = self.mc._replaceTokenMatch(input)
        self.assertEqual(output, 'if (match3(tok, 123) || match4(tok, 123)) {')
        self.assertEqual(2, len(self.mc._matchStrs))

    def test_replaceTokenSimpleMatch(self):
        input = 'if (Token::simpleMatch(tok, "foobar")) {'
        output = self.mc._replaceTokenMatch(input)
        self.assertEqual(output, 'if (match1(tok)) {')
        self.assertEqual(1, len(self.mc._matchStrs))
        self.assertEqual(1, self.mc._matchStrs['foobar'])

        input = 'if (Token::simpleMatch(tok->next()->next(), "foobar")) {'
        output = self.mc._replaceTokenMatch(input)
        self.assertEqual(output, 'if (match1(tok->next()->next())) {')
        self.assertEqual(1, len(self.mc._matchStrs))
        self.assertEqual(1, self.mc._matchStrs['foobar'])

        input = 'if (Token::simpleMatch(tok, "foo\"special\"bar")) {'
        output = self.mc._replaceTokenMatch(input)
        # FIXME: Currently detected as non-static pattern
        self.assertEqual(
            output, 'if (Token::simpleMatch(tok, "foo\"special\"bar")) {')
        self.assertEqual(1, len(self.mc._matchStrs))

    def test_replaceTokenFindSimpleMatch(self):
        input = 'if (Token::findsimplematch(tok, "foobar")) {'
        output = self.mc._replaceTokenFindMatch(input)
        self.assertEqual(output, 'if (findmatch1(tok)) {')
        self.assertEqual(1, len(self.mc._matchStrs))
        self.assertEqual(1, self.mc._matchStrs['foobar'])

        input = 'if (Token::findsimplematch(tok->next()->next(), "foobar", tok->link())) {'
        output = self.mc._replaceTokenFindMatch(input)
        self.assertEqual(
            output, 'if (findmatch2(tok->next()->next(), tok->link())) {')
        self.assertEqual(1, len(self.mc._matchStrs))
        self.assertEqual(1, self.mc._matchStrs['foobar'])

        input = 'if (Token::findsimplematch(tok, "foo\"special\"bar")) {'
        output = self.mc._replaceTokenFindMatch(input)
        # FIXME: Currently detected as non-static pattern
        self.assertEqual(
            output, 'if (Token::findsimplematch(tok, "foo\"special\"bar")) {')
        self.assertEqual(1, len(self.mc._matchStrs))

    def test_replaceTokenFindMatch(self):
        input = 'if (Token::findmatch(tok, "foobar")) {'
        output = self.mc._replaceTokenFindMatch(input)
        self.assertEqual(output, 'if (findmatch1(tok)) {')
        self.assertEqual(1, len(self.mc._matchStrs))
        self.assertEqual(1, self.mc._matchStrs['foobar'])

        # findmatch with varid
        input = 'if (Token::findmatch(tok, "foobar %varid%", tok->varId())) {'
        output = self.mc._replaceTokenFindMatch(input)
        self.assertEqual(output, 'if (findmatch2(tok, tok->varId())) {')
        self.assertEqual(1, len(self.mc._matchStrs))
        self.assertEqual(1, self.mc._matchStrs['foobar'])

        # findmatch with end token
        input = 'if (Token::findmatch(tok->next()->next(), "foobar %type%", tok->link())) {'
        output = self.mc._replaceTokenFindMatch(input)
        self.assertEqual(
            output, 'if (findmatch3(tok->next()->next(), tok->link())) {')
        self.assertEqual(1, len(self.mc._matchStrs))
        self.assertEqual(1, self.mc._matchStrs['foobar'])

        # findmatch with end token and varid
        input = 'if (Token::findmatch(tok->next()->next(), "foobar %type% %varid%", tok->link(), 123)) {'
        output = self.mc._replaceTokenFindMatch(input)
        self.assertEqual(
            output, 'if (findmatch4(tok->next()->next(), tok->link(), 123)) {')
        self.assertEqual(1, len(self.mc._matchStrs))
        self.assertEqual(1, self.mc._matchStrs['foobar'])

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/python

# amazon web services script

import subprocess
import os
import urllib


def wget(url):
    try:
        fp = urllib.urlopen(url)
        data = fp.read()
        return data
    except IOError:
        pass
    return ''

# Perform a git pull.


def gitpull():
    try:
        subprocess.call(['git', 'pull'])

    except IOError:
        pass
    except OSError:
        pass

    return False


def daca2(foldernum):
    folders = '0123456789abcdefghijklmnopqrstuvwxyz'
    folder = folders[foldernum % len(folders)]
    if (foldernum / len(folders)) % 2 == 1:
        folder = 'lib' + folder

    print('Daca2 folder=' + folder)

    p = subprocess.Popen(['git', 'show', '--format=%h'],
                         stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    comm = p.communicate()
    rev = comm[0]
    rev = rev[:rev.find('\n')]

    gitpull()

    subprocess.call(
        ['nice', 'make', 'SRCDIR=build', 'CXXFLAGS=-O2', 'CPPFLAGS=-DMAXTIME=600'])
    subprocess.call(
        ['mv', 'cppcheck', os.path.expanduser('~/daca2/cppcheck-O2')])
    subprocess.call(
        ['cp', 'cfg/std.cfg', os.path.expanduser('~/daca2/')])

    subprocess.call(['python', 'tools/daca2.py', folder, '--rev=' + rev])

    subprocess.call(['cp',
                     os.path.expanduser('~/daca2/' + folder + '/results.txt'),
                     os.path.expanduser('~/daca2/results-' + folder + '.txt')])

subprocess.call(['make', 'clean'])
foldernum = 0
while True:
    daca2(foldernum)
    foldernum = foldernum + 1



#!/usr/bin/python

# continuous integration
# build daily reports (doxygen,coverage,etc)

import datetime
import time
import subprocess
import pexpect
import glob
import os
import sys
import urllib


def wget(url):
    try:
        fp = urllib.urlopen(url)
        data = fp.read()
        return data
    except IOError:
        pass
    return ''


# Upload file to sourceforge web server using scp
def upload(file_to_upload, destination):
    try:
        password = sys.argv[1]
        child = pexpect.spawn(
            'scp ' + file_to_upload + ' danielmarjamaki,cppcheck@web.sourceforge.net:' + destination)
        # child.expect(
        #    'danielmarjamaki,cppcheck@web.sourceforge.net\'s password:')
        child.expect('Password:')
        child.sendline(password)
        child.interact()
    except IOError:
        pass
    except OSError:
        pass
    except pexpect.TIMEOUT:
        pass


# Perform a 'make test' on the repo
def maketest(preclean):
    if preclean == True:
        subprocess.call(['make', 'clean'])

    p = subprocess.Popen(
        ['nice', 'make', 'test'], stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    comm = p.communicate()

    f = open('maketest.txt', 'wt')
    f.write('Errors\n======\n')
    f.write(comm[1] + '\n')
    f.write('Output\n======\n')
    f.write(comm[0] + '\n')
    f.close()

    upload('maketest.txt', 'htdocs/devinfo/')


# git push
def gitpush():
    try:
        password = sys.argv[1]
        child = pexpect.spawn('git push')
        child.expect("Enter passphrase for key '/home/daniel/.ssh/id_rsa':")
        child.sendline(password)
        child.interact()
    except IOError:
        pass
    except OSError:
        pass
    except pexpect.TIMEOUT:
        pass


def iconv(filename):
    p = subprocess.Popen(['file', '-i', filename],
                         stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
    comm = p.communicate()
    if comm[0].find('charset=iso-8859-1') >= 0:
        subprocess.call(
            ["iconv", filename, "--from=ISO-8859-1", "--to=UTF-8", "-o", filename])


# Generate daily webreport
def generate_webreport():
    filenames = glob.glob('*/*.cpp')
    for filename in filenames:
        iconv(filename)
    subprocess.call(
        ["git", "commit", "-a", "-m", '"automatic conversion from iso-8859-1 formatting to utf-8"'])
    gitpush()

    subprocess.call(["rm", "-rf", "devinfo"])
    subprocess.call(['nice', "./webreport.sh"])
    upload('-r devinfo', 'htdocs/')
    subprocess.call(["make", "clean"])
    subprocess.call(["rm", "-rf", "devinfo"])


# Perform a git pull.
def gitpull():
    try:
        password = sys.argv[1]
        child = pexpect.spawn('git pull')
        child.expect("Enter passphrase for key '/home/daniel/.ssh/id_rsa':")
        child.sendline(password)
        child.expect('Already up-to-date.')
        child.interact()

    except IOError:
        pass
    except OSError:
        pass
    except pexpect.TIMEOUT:
        pass
    except pexpect.EOF:
        return True

    return False


def daca2report():
    print('Generate DACA2 report')
    subprocess.call(['rm', '-rf', 'daca2-report'])
    subprocess.call(['mkdir', 'daca2-report'])
    subprocess.call(['python', 'tools/daca2-report.py', 'daca2-report'])
    upload('-r daca2-report', 'htdocs/devinfo/')

def daca2(foldernum):
    folders = '0123456789abcdefghijklmnopqrstuvwxyz'
    folder = folders[foldernum % len(folders)]

    print('Daca2 folder=' + folder)

    p = subprocess.Popen(['git', 'show', '--format=%h'],
                         stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    comm = p.communicate()
    rev = comm[0]
    rev = rev[:rev.find('\n')]

    subprocess.call(
        ['make', 'clean'])
    subprocess.call(
        ['nice', 'make', 'SRCDIR=build', 'CFGDIR=' + os.path.expanduser('~/cppcheck/cfg'), 'CXXFLAGS=-O2', 'CPPFLAGS=-DMAXTIME=600'])
    subprocess.call(
        ['mv', 'cppcheck', os.path.expanduser('~/daca2/cppcheck-O2')])

    subprocess.call(['python', 'tools/daca2.py', folder, '--rev=' + rev])
    daca2report()
    subprocess.call(
        ['python', 'tools/daca2.py', 'lib' + folder, '--rev=' + rev])
    daca2report()

t0 = datetime.date.today()
foldernum = 0
while True:
    if datetime.date.today() != t0:
        print("generate daily reports")
        t0 = datetime.date.today()
        generate_webreport()

    if gitpull() == True:
        print("make test")
        # maketest(False) # Integral make test build

    cmd = wget('http://cppcheck.sourceforge.net/cgi-bin/ci.cgi?clear')
    if cmd.find("doxygen") >= 0:
        generate_webreport()

    daca2(foldernum)
    foldernum = foldernum + 1



# Python module that loads a cppcheck dump
# License: No restrictions, use this as you need.

from lxml import etree

class Token:
    Id               = None
    str              = None
    next             = None
    previous         = None
    scopeId          = None
    scope            = None
    isName           = None
    isNumber         = None
    isInt            = None
    isFloat          = None
    isString         = None
    strlen           = None
    isChar           = None
    isOp             = None
    isArithmeticalOp = None
    isAssignmentOp   = None
    isComparisonOp   = None
    isLogicalOp      = None
    linkId           = None
    link             = None
    varId            = None
    variableId       = None
    variable         = None
    functionId       = None
    function         = None
    valuesId         = None
    values           = None
    astParentId      = None
    astParent        = None
    astOperand1Id    = None
    astOperand1      = None
    astOperand2Id    = None
    astOperand2      = None
    file             = None
    linenr           = None

    def __init__(self, element):
        self.Id            = element.get('id')
        self.str           = element.get('str')
        self.next          = None
        self.previous      = None
        self.scopeId       = element.get('scope')
        self.scope         = None
        type               = element.get('type')
        if type == 'name':
            self.isName = True
        elif type == 'number':
            self.isNumber = True
            if element.get('isInt'):
                self.isInt = True
            elif element.get('isFloat'):
                self.isFloat = True
        elif type == 'string':
            self.isString = True
            self.strlen = int(element.get('strlen'))
        elif type == 'char':
            self.isChar = True
        elif type == 'op':
            self.isOp = True
            if element.get('isArithmeticalOp'):
                self.isArithmeticalOp = True
            elif element.get('isAssignmentOp'):
                self.isAssignmentOp = True
            elif element.get('isComparisonOp'):
                self.isComparisonOp = True
            elif element.get('isLogicalOp'):
                self.isLogicalOp = True
        self.linkId        = element.get('link')
        self.link          = None
        self.varId         = element.get('varId')
        self.variableId    = element.get('variable')
        self.variable      = None
        self.functionId    = element.get('function')
        self.function      = None
        self.valuesId      = element.get('values')
        self.values        = None
        self.astParentId   = element.get('astParent')
        self.astParent     = None
        self.astOperand1Id = element.get('astOperand1')
        self.astOperand1   = None
        self.astOperand2Id = element.get('astOperand2')
        self.astOperand2   = None
        self.file          = element.get('file')
        self.linenr        = element.get('linenr')

    def setId(self, IdMap):
        self.scope       = IdMap[self.scopeId]
        self.link        = IdMap[self.linkId]
        self.variable    = IdMap[self.variableId]
        self.function    = IdMap[self.functionId]
        self.values      = IdMap[self.valuesId]
        self.astParent   = IdMap[self.astParentId]
        self.astOperand1 = IdMap[self.astOperand1Id]
        self.astOperand2 = IdMap[self.astOperand2Id]

    # Get value if it exists
    # Returns None if it doesn't exist
    def getValue(self,v):
        if not self.values:
            return None
        for value in self.values:
            if value.intvalue == v:
                return value
        return None

class Scope:
    Id           = None
    classStartId = None
    classStart   = None
    classEndId   = None
    classEnd     = None
    className    = None
    type         = None

    def __init__(self,element):
        self.Id           = element.get('id')
        self.className    = element.get('className')
        self.classStartId = element.get('classStart')
        self.classStart   = None
        self.classEndId   = element.get('classEnd')
        self.classEnd     = None
        self.nestedInId   = element.get('nestedId')
        self.nestedIn     = None
        self.type         = element.get('type')

    def setId(self, IdMap):
        self.classStart = IdMap[self.classStartId]
        self.classEnd = IdMap[self.classEndId]
        self.nestedIn = IdMap[self.nestedInId]

class Function:
    Id         = None
    argument   = None
    argumentId = None
    def __init__(self,element):
        self.Id         = element.get('id')
        self.argument   = {}
        self.argumentId = {}
        for arg in element:
            self.argumentId[arg.get('nr')] = arg.get('id')
    def setId(self, IdMap):
        for argnr, argid in self.argumentId.iteritems():
            self.argument[argnr] = IdMap[argid]

class Variable:
    Id               = None
    nameTokenId      = None
    nameToken        = None
    typeStartTokenId = None
    typeStartToken   = None
    typeEndTokenId   = None
    typeEndToken     = None
    isArgument       = None
    isArray          = None
    isClass          = None
    isLocal          = None
    isPointer        = None
    isReference      = None
    isStatic         = None

    def __init__(self, element):
        self.Id               = element.get('id')
        self.nameTokenId      = element.get('nameToken')
        self.nameToken        = None
        self.typeStartTokenId = element.get('typeStartToken')
        self.typeStartToken   = None
        self.typeEndTokenId   = element.get('typeEndToken')
        self.typeEndToken     = None
        self.isArgument       = element.get('isArgument')
        self.isArray          = element.get('isArray')
        self.isClass          = element.get('isClass')
        self.isLocal          = element.get('isLocal')
        self.isPointer        = element.get('isPointer')
        self.isReference      = element.get('isReference')
        self.isStatic         = element.get('isStatic')

    def setId(self, IdMap):
        self.nameToken = IdMap[self.nameTokenId]
        self.typeStartToken = IdMap[self.typeStartTokenId]
        self.typeEndToken = IdMap[self.typeEndTokenId]

class ValueFlow:
    class Value:
        intvalue = None
        condition = None
        def __init__(self, element):
            self.intvalue = int(element.get('intvalue'))
            self.condition = element.get('condition-line')
            if self.condition:
                self.condition = int(self.condition)

    Id     = None
    values = None
    def __init__(self, element):
        self.Id  = element.get('id')
        self.values = []
        for value in element:
            self.values.append(ValueFlow.Value(value))

class CppcheckData:
    tokenlist = []
    scopes    = []
    functions = []
    variables = []
    valueflow = []

    def __init__(self, filename):
        self.tokenlist = []
        self.scopes    = []
        self.variables = []
        self.valueflow = []

        data = etree.parse(filename)
        for element in data.getroot():
            if element.tag == 'tokenlist':
                for token in element:
                    self.tokenlist.append(Token(token))

                # set next/previous..
                prev = None
                for token in self.tokenlist:
                    token.previous = prev
                    if prev:
                        prev.next = token
                    prev = token
            if element.tag == 'scopes':
                for scope in element:
                    self.scopes.append(Scope(scope))
                    for functionList in scope:
                        if functionList.tag == 'functionList':
                            for function in functionList:
                                self.functions.append(Function(function))
            if element.tag == 'variables':
                for variable in element:
                    self.variables.append(Variable(variable))
            if element.tag == 'valueflow':
                for values in element:
                    self.valueflow.append(ValueFlow(values))

        IdMap = {}
        IdMap[None] = None
        IdMap['0']  = None
        for token in self.tokenlist:
            IdMap[token.Id] = token
        for scope in self.scopes:
            IdMap[scope.Id] = scope
        for function in self.functions:
            IdMap[function.Id] = function
        for variable in self.variables:
            IdMap[variable.Id] = variable
        for values in self.valueflow:
            IdMap[values.Id] = values.values

        for token in self.tokenlist:
            token.setId(IdMap)
        for scope in self.scopes:
            scope.setId(IdMap)
        for function in self.functions:
            function.setId(IdMap)
        for variable in self.variables:
            variable.setId(IdMap)

def parsedump(filename):
    return CppcheckData(filename)

# Check if type of ast node is float/double
def astIsFloat(token):
    if not token:
        return False
    if token.str == '.':
        return astIsFloat(token.astOperand2)
    if '+-*/%'.find(token.str) == 0:
        if True == astIsFloat(token.astOperand1):
            return True
        return astIsFloat(token.astOperand2)
    if not token.variable:
        # float literal?
        if token.str[0].isdigit():
            for c in token.str:
                if c=='f' or c=='.' or c=='E':
                    return True
        return False
    typeToken = token.variable.typeStartToken
    endToken = token.variable.typeEndToken
    while typeToken != endToken:
        if typeToken.str == 'float' or typeToken.str == 'double':
            return True
        typeToken = typeToken.next
    if typeToken.str == 'float' or typeToken.str == 'double':
        return True
    return False
